package exercises.ex_z4;

import java.util.concurrent.ConcurrentLinkedQueue;
import java.util.concurrent.Semaphore;

public class Auctioneer implements Runnable {
    private String name;
    private ConcurrentLinkedQueue<Bid> bids;
    private Semaphore mutex;
    private Bid maximumBid;
    private Thread countdownThread = null;

    public Auctioneer(String name, ConcurrentLinkedQueue<Bid> bids, Semaphore mutex) {
        this.name = name;
        this.bids = bids;
        this.mutex = mutex;
    }

    public void placeBid(Bid bid) {
        bids.add(bid);
        notify();
    }

    private void countdown() {
        try {
            Thread.sleep(1000);
            System.out.println("Three");
            Thread.sleep(1000);
            System.out.println("Two");
            Thread.sleep(1000);
            System.out.println("One");
            Thread.sleep(1000);
            System.out.println("End of the Auction !");
            notify();
        } catch (InterruptedException _) {}
    }

    public void startCountdown() {
        countdownThread = new Thread(this::countdown);
        countdownThread.start();
    }

    public Bid getMaximumBid() {
        return maximumBid;
    }

    public void endAuction(){
        System.out.println("End of the auction ! ");
        notifyAll();
    }

    @Override
    public void run() {
        while (true) {
            try {
                mutex.acquire();
                Bid current_bid = bids.poll();
                if (current_bid == null) {
                    startCountdown();
                    wait();
                    if (bids.isEmpty()) {
                        endAuction();
                    }
                    continue;
                }
                if (!bids.isEmpty()) {
                    if (current_bid.amount() > maximumBid.amount()) {
                        maximumBid = current_bid;
                        System.out.println("New bid : " + maximumBid.amount() + " by " + maximumBid.bidder());
                        mutex.release();
                        return;
                    }
                } else {
                    mutex.release();
                }
            } catch (InterruptedException e) {
                throw new RuntimeException(e);
            }
        }
    }
}
